import * as mc from "@minecraft/server";
import { dnaExtractorInteractEvent } from "./dnaExtractor";
import { centrifugeScriptEvent } from "./machines/centrifuge";
import { poisonSpray, spawnXP, triggerDinosaurRoar } from "./utilities";
import { amberConverterScriptEvent, dnaExtractUseEvent } from "./machines/amberConverter";
import { dinoEggUseEvent, incubatorScriptEvent } from "./machines/incubator";
import { electricGunShootEvent, tranquilizingSniperHitEvent, tranquilizingSniperShootEvent } from "./guns";
import { NAMESPACE } from "./anvilConstants";
import { initSyringe, spawnFullSyringes, syringeManager } from "./syringesManager/syringeManager";
import { damageMorph, morphDeath, morphTick } from "./morphManager/morphManager";
import { playerSpawn } from "./morphManager/morphPlayer";
import { morphActionManager } from "./morphManager/morphActions";
import { backpackPlaceEvent, saveBackpackContainer } from "./backpack/backpackManager";
const players = mc.world.getAllPlayers().reduce((map, player) => {
    map.set(player.id, player);
    return map;
}, new Map());
mc.world.afterEvents.playerSpawn.subscribe((event) => {
    if (event.initialSpawn) {
        players.set(event.player.id, event.player);
        playerSpawn(event);
    }
});
mc.world.afterEvents.playerLeave.subscribe((event) => {
    players.delete(event.playerId);
});
// Events -----------------------------------------------------------------------
mc.world.afterEvents.entityDie.subscribe((eventData) => {
    const damagingEntity = eventData.damageSource.damagingEntity;
    if (damagingEntity) {
        triggerDinosaurRoar(damagingEntity, eventData.deadEntity);
    }
});
mc.world.afterEvents.itemStartUse.subscribe((eventData) => {
    dnaExtractorInteractEvent(eventData);
    dnaExtractUseEvent(eventData);
    dinoEggUseEvent(eventData);
    syringeManager(eventData);
    morphActionManager(eventData);
});
mc.world.afterEvents.itemReleaseUse.subscribe((eventData) => {
    electricGunShootEvent(eventData);
    tranquilizingSniperShootEvent(eventData);
});
mc.world.afterEvents.itemCompleteUse.subscribe((eventData) => {
    if (eventData.itemStack.typeId === `${NAMESPACE}:cooked_dino_meat`) {
        eventData.source.addEffect("resistance", 10 * 20, { amplifier: 1 });
        eventData.source.addEffect("strength", 10 * 20, { amplifier: 1 });
    }
});
mc.world.afterEvents.projectileHitEntity.subscribe((eventData) => {
    tranquilizingSniperHitEvent(eventData);
    damageMorph(eventData.getEntityHit().entity);
    spawnXP(eventData.getEntityHit().entity, eventData.source);
});
mc.world.afterEvents.entityHitEntity.subscribe((eventData) => {
    damageMorph(eventData.hitEntity);
    spawnXP(eventData.hitEntity, eventData.damagingEntity);
});
mc.world.afterEvents.entityHealthChanged.subscribe((eventData) => {
    morphDeath(eventData);
});
mc.world.beforeEvents.itemUseOn.subscribe((eventData) => {
    const itemStack = eventData.itemStack;
    if (itemStack && itemStack.typeId === `${NAMESPACE}:pteranodon_trophy_spawn_egg`) {
        if (eventData.blockFace !== mc.Direction.Down ||
            !(eventData.block.below(1)?.permutation.matches("minecraft:air") &&
                eventData.block.below(2)?.permutation.matches("minecraft:air") &&
                eventData.block.below(3)?.permutation.matches("minecraft:air") &&
                eventData.block.below(4)?.permutation.matches("minecraft:air"))) {
            eventData.cancel = true;
        }
    }
});
mc.world.afterEvents.itemUseOn.subscribe((eventData) => {
    backpackPlaceEvent(eventData);
});
mc.world.afterEvents.playerBreakBlock.subscribe((eventData) => {
    const below = eventData.block.below(1);
    if (below) {
        below.dimension
            .getEntities({ location: below?.location })
            .find((entity) => {
                return entity.typeId === `${NAMESPACE}:pteranodon_trophy`;
            })
            ?.triggerEvent("drop_self");
    }
});
mc.system.afterEvents.scriptEventReceive.subscribe((eventData) => {
    centrifugeScriptEvent(eventData);
    amberConverterScriptEvent(eventData);
    incubatorScriptEvent(eventData);
    spawnFullSyringes(eventData);
    poisonSpray(eventData);
});
mc.system.afterEvents.scriptEventReceive.subscribe((data) => {
    saveBackpackContainer(data);
});
["gamerule showtags false", "gamerule sendcommandfeedback false", "gamerule commandblockoutput false"].forEach((command) => {
    mc.world.getDimension(mc.MinecraftDimensionTypes.overworld).runCommand(command);
});
mc.system.runInterval(() => {
    for (const player of players.values()) {
        initSyringe(player);
        morphTick(player);
    }
});
// runs in a random interval of 0 to 5 seconds, every 20 minutes for 1 minute
//mc.system.runInterval(() => {
//	const tick = mc.system.currentTick;
//
//	const interval = 20;
//	const duration = 1;
//
//	const intervalTick = interval * 60 * 20;
//	const durationTick = duration * 60 * 20;
//
//	if (tick % intervalTick >= 0 && tick % intervalTick <= durationTick) {
//		meteorEvent();
//	}
//}, Math.random() * 5 * 20);
//
